/**
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package frame;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import org.netbeans.api.visual.widget.Scene;

/**
 * Classe de uma Qdisc do tipo ClassFull
 *
 * @author Luiz Gustavo Ender
 */
public class ClassFul extends Qdisc {

    private Classe cdefault;
    private String auxCdefault;
    private ArrayList<Classe> classes;
    private JTextField jTFdefault, jTFparent;

    public ClassFul(Scene scene) {
        super(scene);
        auxCdefault = "";
        classes = new ArrayList<Classe>();
        jTFdefault = new JTextField();
        jTFparent = new JTextField();
    }

    public void addClasse(Classe c) {
        classes.add(c);
    }

    public void rmClasse(Classe c) {
        classes.remove(c);
    }

    public String getCdefaultString() {
        if (cdefault != null) {
            return cdefault.getClassid();
        }
        return auxCdefault;
    }

    public Classe getCdefault() {
        return cdefault;
    }

    public void setCdefault(Classe cdefault) {
        this.cdefault = cdefault;
    }

    public void atualizaCdefault() {
        this.cdefault = (Classe) ((VisualScene) getScene()).getParent(auxCdefault);
    }

    public void setCdefault(String id) {
        this.auxCdefault = id;
    }

    public ArrayList<Classe> getClasses() {
        return classes;
    }

    /**
     * Função que criar o painel de propriedades do objeto.
     *
     * @return retorna um JDialog que será o painel.
     */
    @Override
    public void geraPainelConf() {
        jDconf = new JDialog(((VisualScene) getScene()).getFrame(), true);
        jDconf.setTitle("Configurações");
        jDconf.setLayout(null);

        int y = 10;

        JLabel jLabel = new JLabel("Interface:");
        jLabel.setBounds(10, y, 60, 19);
        jDconf.add(jLabel);
        jTFiface.setText(((VisualScene) getScene()).getIface());
        jTFiface.setEditable(false);
        jTFiface.setBounds(75, y, 110, 25);
        jDconf.add(jTFiface);
        jLabel = new JLabel(new ImageIcon(getClass().getResource("/imagens/duvida2.gif")));
        jLabel.setToolTipText("<html>"
                + "Mostra a interface de rede utilizada." + "<br>"
                + "</html>");
        jLabel.setBounds(195, y + 4, 16, 16);
        jDconf.add(jLabel);
        y = jTFiface.getY() + 25;

        if (getImagemNome().equals("qdisc_sub.png")) {
            jLabel = new JLabel("Parent:");
            jLabel.setBounds(10, y, 60, 19);
            jDconf.add(jLabel);
            String p = "";
            if (parent instanceof Qdisc) {
                p = ((Qdisc) parent).getHandle();
            } else if (parent instanceof Classe) {
                p = ((Classe) parent).getClassid();
            }
            jTFparent.setDocument(new MaskField(1));
            jTFparent.setText(p);
            jTFparent.setBounds(75, y, 110, 25);
            jDconf.add(jTFparent);
            jLabel = new JLabel(new ImageIcon(getClass().getResource("/imagens/duvida2.gif")));
            jLabel.setToolTipText("<html>"
                    + "Define a classe/qdisc 'pai' deste componente." + "<br>"
                    + "&nbsp; - Exemplo: 1:0"
                    + "</html>");
            jLabel.setBounds(195, y + 4, 16, 16);
            jDconf.add(jLabel);
            y = jTFparent.getY() + 25;
        }

        jLabel = new JLabel("Handle:");
        jLabel.setBounds(10, y, 60, 19);
        jDconf.add(jLabel);
        jTFhandle.setEditable(false);
        jTFhandle.setText(handle);
        jTFhandle.setBounds(75, y, 110, 25);
        jDconf.add(jTFhandle);
        jLabel = new JLabel(new ImageIcon(getClass().getResource("/imagens/duvida2.gif")));
        jLabel.setToolTipText("<html>"
                + "Mostra o nome do componente." + "<br>"
                + "</html>");
        jLabel.setBounds(195, y + 4, 16, 16);
        jDconf.add(jLabel);
        y = jTFhandle.getY() + 25;

        jLabel = new JLabel("Qdisc: ");
        jLabel.setBounds(10, y, 60, 19);
        jDconf.add(jLabel);
        jTFqdisc.setText(qdisc);
        jTFqdisc.setBounds(75, y, 110, 25);
        jTFqdisc.setEditable(false);
        jDconf.add(jTFqdisc);
        jLabel = new JLabel(new ImageIcon(getClass().getResource("/imagens/duvida2.gif")));
        jLabel.setToolTipText("<html>"
                + "Mostra o tipo de disciplina de fila que será utilizado." + "<br>"
                + "</html>");
        jLabel.setBounds(195, y + 4, 16, 16);
        jDconf.add(jLabel);
        y = jTFqdisc.getY() + 25;

        if ("HTB".equalsIgnoreCase(qdisc)) {
            jLabel = new JLabel("Default: ");
            jLabel.setBounds(10, y, 60, 19);
            jDconf.add(jLabel);
            jTFdefault.setDocument(new MaskField(1));
            if (cdefault != null) {
                jTFdefault.setText(getCdefaultString());
                auxCdefault = cdefault.getClassid();
            } else {
                jTFdefault.setText(auxCdefault);
            }
            jTFdefault.setBounds(75, y, 110, 25);
            jDconf.add(jTFdefault);
            jLabel = new JLabel(new ImageIcon(getClass().getResource("/imagens/duvida2.gif")));
            jLabel.setToolTipText("<html>"
                    + "Define a classe/qdisc default que os pacotes serão encaminhados." + "<br>"
                    + "&nbsp; - Exemplo: 2:10"
                    + "</html>");
            jLabel.setBounds(195, y + 4, 16, 16);
            jDconf.add(jLabel);
            y = jTFdefault.getY() + 25;
        }

        JButton jBsalvar = new JButton("Salvar");
        jBsalvar.setBounds(115, y, 70, 25);
        jBsalvar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                jBsalvar(e);
            }
        });
        jDconf.add(jBsalvar);
        y = jBsalvar.getY() + 25 + 30;

        jDconf.setBounds(1, 1, 220, y);
        jDconf.setSize(220, y);
        jDconf.setLocationRelativeTo(null);
        jDconf.setVisible(true);
    }

    /**
     * Evento do botão salvar
     *
     * @param evt evento
     */
    public void jBsalvar(ActionEvent evt) {
        handle = jTFhandle.getText();

        if (!"".equals(jTFdefault.getText())) {
            Classe c = (Classe) ((VisualScene) getScene()).getParent(jTFdefault.getText());
            if (c != null) {
                cdefault = c;
                auxCdefault = c.getClassid();
            } else {
                JOptionPane.showMessageDialog(jDconf, "Não foi encontrado a classe " + jTFdefault.getText() + ".",
                        "Erro", JOptionPane.ERROR_MESSAGE);
            }
        }

        if (getImagemNome().equals("qdisc_sub.png")) {
            if (!"".equals(jTFparent.getText())) {
                if (parent == null) {
                    parent = ((VisualScene) getScene()).getParent(jTFparent.getText());
                    if (parent != null) {
                        ((VisualScene) getScene()).getSCP().createConnection(parent, this);
                    }
                } else if (!jTFparent.getText().equals(parent.getLabelWidget().getLabel())) {
                    ((VisualScene) getScene()).getSRP().setEdge(((VisualScene) getScene()).getEdge(parent, this));
                    parent = ((VisualScene) getScene()).getParent(jTFparent.getText());
                    if (parent != null) {
                        ((VisualScene) getScene()).getSRP().reconnect(null, parent, true);
                    }
                }
            }
        }
        jDconf.dispose();
    }

    /**
     * Função que gera o comando TC para um objeto.
     *
     * @return retorna o comando TC.
     */
    @Override
    public String geraComandosTC() {
        String comando = "tc qdisc add dev " + ((VisualScene) getScene()).getIface();
        if (getImagemNome().equals("qdisc.png")) {
            comando = comando + " root handle " + handle + " " + qdisc.toLowerCase();
            if (qdisc.equalsIgnoreCase("htb")) {
                if (cdefault != null) {
                    if (!"".equals(getCdefaultString())) {
                        comando = comando + " default " + ((VisualScene) getScene()).getSufixo(getCdefaultString());
                    }
                } else if (!"".equals(auxCdefault)) {
                    comando = comando + " default " + ((VisualScene) getScene()).getSufixo(auxCdefault);
                }
            }
        } else {
            if (parent != null) {
                comando = comando + " parent " + getParentString() + " handle " + handle + " " + qdisc.toLowerCase();
            }
            if (qdisc.equalsIgnoreCase("htb")) {
                if (cdefault != null) {
                    if (!"".equals(getCdefaultString())) {
                        comando = comando + " default " + ((VisualScene) getScene()).getSufixo(getCdefaultString());
                    }
                } else if (!"".equals(auxCdefault)) {
                    comando = comando + " default " + ((VisualScene) getScene()).getSufixo(auxCdefault);
                }
            }
        }
        return comando;
    }
}
