/**
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package frame;

import java.awt.Point;
import java.util.HashSet;
import java.util.Set;
import javax.swing.JOptionPane;
import org.netbeans.api.visual.action.ConnectProvider;
import org.netbeans.api.visual.action.ConnectorState;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;

/**
 * Função que criar a conexão entre dois objetos
 *
 * @author Luiz Gustavo Ender
 */
public class MyConnectProvider implements ConnectProvider {

    private ObjetoNode source;
    private ObjetoNode target;
    private VisualScene scene;

    public MyConnectProvider(VisualScene scene) {
        this.scene = scene;
        this.source = null;
        this.target = null;
    }

    @Override
    public boolean isSourceWidget(Widget w) {
        Object object = scene.findObject(w);
        source = scene.isNode(object) ? (ObjetoNode) object : null;
        return source != null;
    }

    @Override
    public ConnectorState isTargetWidget(Widget w, Widget w1) {
        Object object = scene.findObject(w1);
        target = scene.isNode(object) ? (ObjetoNode) object : null;
        if (target != null) {
            return !source.equals(target) ? ConnectorState.ACCEPT : ConnectorState.REJECT_AND_STOP;
        }
        return object != null ? ConnectorState.REJECT_AND_STOP : ConnectorState.REJECT;
    }

    @Override
    public boolean hasCustomTargetWidgetResolver(Scene scene) {
        return false;
    }

    @Override
    public Widget resolveTargetWidget(Scene scene, Point point) {
        return null;
    }

    @Override
    public void createConnection(Widget w, Widget w1) {
        if ((w instanceof Objeto) && (w1 instanceof Objeto)) {
            isSourceWidget(w);
            isTargetWidget(w, w1);
        }
        Objeto oSource = (Objeto) scene.findWidget(source);
        Objeto oTarget = (Objeto) scene.findWidget(target);

        // Verifica se atingiu o nivel máximo
        if (oSource.getNivel() == 7) {
            JOptionPane.showMessageDialog(scene.getFrame(), "Você atingiu o número máximo de níveis!",
                    "Erro", JOptionPane.ERROR_MESSAGE);
        } else {
            // Verifica se a conexão ja existe
            if (scene.isExisteEdge(oSource, oTarget)) {
                JOptionPane.showMessageDialog(scene.getFrame(), "Já existe esta conexão!",
                        "Erro", JOptionPane.ERROR_MESSAGE);
            } else {
                boolean criaEdge = scene.isCreateConnection(oSource, oTarget);
                // Verifica se pode criar essa conexão
                if (criaEdge) {
                    if ((oSource instanceof ClassLess) && (oTarget instanceof Classe)) {
                        JOptionPane.showMessageDialog(scene.getFrame(), "Uma Qdisc do tipo ClassLess não pode conter Classe.",
                                "Erro", JOptionPane.ERROR_MESSAGE);
                    } else {
                        String label = "";
                        oSource.getChild().add(oTarget);
                        // Verifica se o target é uma ClassFul
                        if (oTarget instanceof ClassFul) {
                            oTarget.setParent(oSource);
                            // Verifica se o source é um ClassFul
                            if (oSource instanceof ClassFul) {
                                label = ((ClassFul) oSource).getHandle();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((ClassFul) oTarget).setHandle(label);
                            }
                            // Verifica se o source é uma Classe
                            if (oSource instanceof Classe) {
                                label = ((Classe) oSource).getClassid();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((ClassFul) oTarget).setHandle(label);
                                // Verifica se já existe uma Qdisc associada a está classe.
                                if (((Classe) oSource).getQdisc() == null) {
                                    ((Classe) oSource).setQdisc((ClassFul) oTarget);
                                } else {
                                    int aux = JOptionPane.showConfirmDialog(scene.getFrame(), "Já existe uma Qdisc associada a esta Classe\n"
                                            + "Deseja substituir?",
                                            "Aviso", JOptionPane.YES_NO_OPTION);
                                    if (aux == JOptionPane.YES_OPTION) {
                                        String edge = scene.getEdge(oSource, ((Classe) oSource).getQdisc());
                                        Set<String> select = new HashSet<String>();
                                        select.add(edge);
                                        if (select.size() > 0) {
                                            scene.setSelectedObjects(select);
                                            scene.deleteSelectedNodes(true);
                                        }
                                        if (edge != null) {
                                            scene.removeEdge(edge);
                                        }
                                        if (((Classe) oSource).getQdisc() != null) {
                                            ((Classe) oSource).getQdisc().setHandle("");
                                            ((Classe) oSource).getQdisc().setParent((Objeto) null);
                                            ((Classe) oSource).getQdisc().setLabel("");
                                        }
                                        ((Classe) oSource).setQdisc((ClassFul) oTarget);
                                    } else {
                                        return;
                                    }
                                }
                            }
                            oTarget.setLabel(label);
                            // Verifica se o target é um ClassLess
                        } else if (oTarget instanceof ClassLess) {
                            oTarget.setParent(oSource);
                            if (oSource instanceof ClassFul) {
                                label = ((ClassFul) oSource).getHandle();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((ClassLess) oTarget).setHandle(label);
                            } else if (oSource instanceof ClassLess) {
                                label = ((ClassLess) oSource).getHandle();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((ClassLess) oTarget).setHandle(label);
                            } else if (oSource instanceof Classe) {
                                if (((Classe) oSource).getQdisc() == null) {
                                    ((Classe) oSource).setQdisc((ClassLess) oTarget);
                                } else {
                                    int aux = JOptionPane.showConfirmDialog(scene.getFrame(), "Já existe uma Qdisc associada a esta Classe\n"
                                            + "Deseja substituir?",
                                            "Aviso", JOptionPane.YES_NO_OPTION);
                                    if (aux == JOptionPane.YES_OPTION) {
                                        String edge = scene.getEdge(oSource, ((Classe) oSource).getQdisc());
                                        Set<String> select = new HashSet<String>();
                                        select.add(edge);
                                        if (select.size() > 0) {
                                            scene.setSelectedObjects(select);
                                            scene.deleteSelectedNodes(true);
                                        }
                                        if (edge != null) {
                                            scene.removeEdge(edge);
                                        }
                                        if (((Classe) oSource).getQdisc() != null) {
                                            ((Classe) oSource).getQdisc().setHandle("");
                                            ((Classe) oSource).getQdisc().setParent((Objeto) null);
                                            ((Classe) oSource).getQdisc().setLabel("");
                                        }
                                        ((Classe) oSource).setQdisc((ClassLess) oTarget);
                                    } else {
                                        oSource.getChild().remove(oTarget);
                                        return;
                                    }
                                }
                                label = ((Classe) oSource).getClassid();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((ClassLess) oTarget).setHandle(label);
                            }
                            oTarget.setLabel(label);
                        } else if (oTarget instanceof Classe) {
                            oTarget.setParent(oSource);
                            if (oSource instanceof ClassFul) {
                                ((ClassFul) oSource).addClasse((Classe) oTarget);
                                for (Objeto o : oTarget.getChild()) {
                                    if (o instanceof Classe) {
                                        ((ClassFul) oSource).addClasse((Classe) o);
                                    }
                                }
                                if (((Classe) oTarget).getNomClass().equals("prio")) {
                                    if (oTarget.getParentString() != null) {
                                        label = oTarget.getParentString() + ((Classe) oTarget).getBand();
                                    }
                                } else {
                                    label = ((ClassFul) oSource).getHandle();
                                    label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                }
                                oTarget.setNivel(oSource.getNivel() + 1);
                                ((Classe) oTarget).setClassid(label);
                            } else if (oSource instanceof Classe) {
                                ClassFul cl = scene.getClassFul(oSource);
                                if (cl != null) {
                                    cl.addClasse((Classe) oTarget);
                                    label = ((Classe) oSource).getClassid();
                                    oTarget.setNivel(oSource.getNivel() + 1);
                                    label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                    ((Classe) oTarget).setClassid(label);
                                } else {
                                    JOptionPane.showMessageDialog(scene.getFrame(), "Não é possível criar esta conexão, pois não foi \n"
                                            + "encontrado uma Qdisc do tipo ClassFull\n"
                                            + "associada ao objeto de oriem desta conexão.",
                                            "Erro", JOptionPane.ERROR_MESSAGE);
                                    oSource.getChild().remove(oTarget);
                                    return;
                                }
                            }
                            oTarget.setLabel(label);
                        }
                        int aux = scene.getEdgeIDcounter() + 1;
                        source.setEdge(aux + "1");
                        target.setEdge(aux + "2");
                        scene.addEdge("" + aux);
                        scene.setEdgeSource("" + aux, source);
                        scene.setEdgeTarget("" + aux, target);
                        scene.setEdgeIDcounter(aux);
                        scene.updateChildren(oTarget);
                        scene.validate();
                    }
                } else {
                    JOptionPane.showMessageDialog(scene.getFrame(), "Não é possível criar esta conexão!",
                            "Erro", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }
}
