/**
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package frame;

import java.awt.Point;
import java.util.HashSet;
import java.util.Set;
import javax.swing.JOptionPane;
import org.netbeans.api.visual.action.ConnectorState;
import org.netbeans.api.visual.action.ReconnectProvider;
import org.netbeans.api.visual.widget.ConnectionWidget;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;

/**
 * Função que cria a reconexão.
 *
 * @author Luiz Gustavo Ender
 */
public class MyReconnectProvider implements ReconnectProvider {
    
    private String edge;
    private ObjetoNode originalNode;
    private ObjetoNode replacementNode;
    private VisualScene scene;
    
    public MyReconnectProvider(VisualScene scene) {
        this.scene = scene;
    }
    
    @Override
    public boolean isSourceReconnectable(ConnectionWidget cw) {
        Object object = scene.findObject(cw);
        edge = scene.isEdge(object) ? (String) object : null;
        originalNode = edge != null ? (ObjetoNode) scene.getEdgeSource(edge) : null;
        return originalNode != null;
    }
    
    @Override
    public boolean isTargetReconnectable(ConnectionWidget cw) {
        Object object = scene.findObject(cw);
        edge = scene.isEdge(object) ? (String) object : null;
        originalNode = edge != null ? (ObjetoNode) scene.getEdgeTarget(edge) : null;
        return originalNode != null;
    }
    
    @Override
    public void reconnectingStarted(ConnectionWidget cw, boolean bln) {
    }
    
    @Override
    public void reconnectingFinished(ConnectionWidget cw, boolean bln) {
    }
    
    @Override
    public ConnectorState isReplacementWidget(ConnectionWidget cw, Widget w, boolean bln) {
        Object object = scene.findObject(w);
        replacementNode = scene.isNode(object) ? (ObjetoNode) object : null;
        if (replacementNode != null) {
            return ConnectorState.ACCEPT;
        }
        return object != null ? ConnectorState.REJECT_AND_STOP : ConnectorState.REJECT;
    }
    
    @Override
    public boolean hasCustomReplacementWidgetResolver(Scene scene) {
        return false;
    }
    
    @Override
    public Widget resolveReplacementWidget(Scene scene, Point point) {
        return null;
    }
    
    @Override
    public void reconnect(ConnectionWidget cw, Widget w, boolean reconnectSource) {
        Objeto oSource, oTarget;
        
        if (reconnectSource) {
            if (w instanceof Objeto) {
                isReplacementWidget(cw, w, reconnectSource);
            }
            oSource = (Objeto) scene.findWidget(replacementNode);
            oTarget = (Objeto) scene.findWidget(scene.getEdgeTarget(edge));
        } else {
            oSource = (Objeto) scene.findWidget(scene.getEdgeSource(edge));
            if (w instanceof Objeto) {
                isReplacementWidget(cw, w, reconnectSource);
            }
            oTarget = (Objeto) scene.findWidget(replacementNode);
        }
        Objeto sourceAntigo = (Objeto) scene.findWidget(scene.getEdgeSource(edge));
        Objeto targetAntigo = (Objeto) scene.findWidget(scene.getEdgeTarget(edge));

        // Verifica se o usuário não deu um loop na hierarquia
        boolean criaEdge = scene.isCreateConnection(oSource, oTarget);
        if (criaEdge) {
            Set<Objeto> cascata = new HashSet<Objeto>();
            if (reconnectSource) {
                cascata.addAll(scene.getChildrenObjeto(sourceAntigo));
                if (cascata.contains(oSource)) {
                    criaEdge = false;
                }
            } else {
                cascata.addAll(scene.getChildrenObjeto(sourceAntigo));
                if (cascata.contains(oTarget)) {
                    criaEdge = false;
                }
            }
        }
        if ((oSource != null) && (oTarget != null)) {
            if (criaEdge) {
                if ((oSource instanceof ClassLess) && (oTarget instanceof Classe)) {
                    JOptionPane.showMessageDialog(scene.getFrame(), "Uma Qdisc do tipo ClassLess não pode conter Classe.",
                            "Erro", JOptionPane.ERROR_MESSAGE);
                } else {
                    String label = "";
                    if (!reconnectSource) {
                        targetAntigo.setNivel(0);
                    }
                    if (oTarget instanceof ClassFul) {
                        oTarget.setParent(oSource);
                        if (oSource instanceof Classe) {
                            label = ((Classe) oSource).getClassid();
                            oTarget.setNivel(oSource.getNivel() + 1);
                            label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                            if (reconnectSource == false) {
                                ((Classe) oSource).setQdisc(null);
                            }
                            if (((Classe) oSource).getQdisc() == null) {
                                ((Classe) oSource).setQdisc((ClassFul) oTarget);
                                ((Classe) sourceAntigo).setQdisc(null);
                            } else {
                                int aux = JOptionPane.showConfirmDialog(scene.getFrame(), "Já existe uma Qdisc associada a esta Classe\n"
                                        + "Deseja substituir?",
                                        "Aviso", JOptionPane.YES_NO_OPTION);
                                if (aux == JOptionPane.YES_OPTION) {
                                    String edge = scene.getEdge(oSource, ((Classe) oSource).getQdisc());
                                    Set<String> select = new HashSet<String>();
                                    select.add(edge);
                                    scene.setSelectedObjects(select);
                                    scene.deleteSelectedNodes(true);
                                    if (edge != null) {
                                        scene.removeEdge(edge);
                                    }
                                    if (((Classe) oSource).getQdisc() != null) {
                                        ((Classe) oSource).getQdisc().setHandle("");
                                        ((Classe) oSource).getQdisc().setParent((Objeto) null);
                                        ((Classe) oSource).getQdisc().setLabel("");
                                    }
                                    ((Classe) oSource).setQdisc((ClassLess) oTarget);
                                }
                            }
                        }
                    } else if (oTarget instanceof ClassLess) {
                        oTarget.setParent(oSource);
                        if (oSource instanceof ClassFul) {
                            label = ((ClassFul) oSource).getHandle();
                            oTarget.setNivel(oSource.getNivel() + 1);
                            label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                            ((ClassLess) oTarget).setHandle(label);
                        } else if (oSource instanceof ClassLess) {
                            label = ((ClassLess) oSource).getHandle();
                            oTarget.setNivel(oSource.getNivel() + 1);
                            label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                            ((ClassLess) oTarget).setHandle(label);
                        } else if (oSource instanceof Classe) {
                            label = ((Classe) oSource).getClassid();
                            oTarget.setNivel(oSource.getNivel() + 1);
                            label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                            ((ClassLess) oTarget).setHandle(label);
                            if (reconnectSource == false) {
                                ((Classe) oSource).setQdisc(null);
                            }
                            if (((Classe) oSource).getQdisc() == null) {
                                ((Classe) oSource).setQdisc((ClassLess) oTarget);
                                ((Classe) sourceAntigo).setQdisc(null);
                            } else {
                                int aux = JOptionPane.showConfirmDialog(scene.getFrame(), "Já existe uma Qdisc associada a esta Classe\n"
                                        + "Deseja substituir?",
                                        "Aviso", JOptionPane.YES_NO_OPTION);
                                if (aux == JOptionPane.YES_OPTION) {
                                    String edge = scene.getEdge(oSource, ((Classe) oSource).getQdisc());
                                    Set<String> select = new HashSet<String>();
                                    select.add(edge);
                                    scene.setSelectedObjects(select);
                                    scene.deleteSelectedNodes(true);
                                    if (edge != null) {
                                        scene.removeEdge(edge);
                                    }
                                    if (((Classe) oSource).getQdisc() != null) {
                                        ((Classe) oSource).getQdisc().setHandle("");
                                        ((Classe) oSource).getQdisc().setParent((Objeto) null);
                                        ((Classe) oSource).getQdisc().setLabel("");
                                    }
                                    ((Classe) oSource).setQdisc((ClassLess) oTarget);
                                }
                            }
                        }
                        oTarget.setLabel(label);
                    } else if (oTarget instanceof Classe) {
                        oTarget.setParent(oSource);
                        if (oSource instanceof ClassFul) {
                            ((ClassFul) oSource).rmClasse((Classe) targetAntigo);
                            ((ClassFul) oSource).addClasse((Classe) oTarget);
                            label = ((ClassFul) oSource).getHandle();
                            oTarget.setNivel(oSource.getNivel() + 1);
                            label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                            ((Classe) oTarget).setClassid(label);
                        } else if (oSource instanceof Classe) {
                            ClassFul cf = scene.getClassFul(oSource);
                            if (cf != null) {
                                cf.rmClasse((Classe) targetAntigo);
                                cf.addClasse((Classe) oTarget);
                                label = ((Classe) oSource).getClassid();
                                oTarget.setNivel(oSource.getNivel() + 1);
                                label = scene.defineLabel(oSource.getNivel() + 1, label, oTarget);
                                ((Classe) oTarget).setClassid(label);
                            } else {
                                targetAntigo.setNivel(oSource.getNivel() + 1);
                                JOptionPane.showMessageDialog(scene.getFrame(), "Não é possível criar esta conexão, pois não foi \n"
                                        + "encontrado uma Qdisc do tipo ClassFull\n"
                                        + "associada ao objeto de oriem desta conexão.",
                                        "Erro", JOptionPane.ERROR_MESSAGE);
                                return;
                            }
                        }
                        oTarget.setLabel(label);
                    }
                    if (w == null) {
                        scene.removeEdge(edge);
                    } else if (reconnectSource) {
                        scene.setEdgeSource(edge, replacementNode);
                        sourceAntigo.getChild().remove(oTarget);
                        oSource.getChild().add(oTarget);
                    } else {
                        boolean delCascata = true;
                        ObjetoNode on = (ObjetoNode) scene.findObject(targetAntigo);
                        for (String s : scene.getEdges()) {
                            if (!s.equals(edge)) {
                                if (scene.getEdgeTarget(s) == on) {
                                    delCascata = false;
                                }
                            }
                        }
                        if (delCascata) {
                            targetAntigo.setNivel(0);
                            targetAntigo.setLabel("");
                            if (targetAntigo instanceof Qdisc){
                                ((Qdisc) targetAntigo).setHandle("");
                            } else if (targetAntigo instanceof Classe){
                                ((Classe) targetAntigo).setClassid("");
                            }
                            Set<String> select = new HashSet<String>();
                            select.add(edge);
                            scene.setSelectedObjects(select);
                            scene.deleteSelectedNodes(true);
                        }
                        oSource.getChild().add(oTarget);
                        oSource.getChild().remove(targetAntigo);
                        scene.setEdgeTarget(edge, replacementNode);
                        scene.updateChildren(oTarget);
                    }
                }
            } else {
                JOptionPane.showMessageDialog(scene.getFrame(), "Não é possível criar esta conexão!",
                        "Erro", JOptionPane.ERROR_MESSAGE);
            }
            scene.validate();
            scene.repaint();
        }
    }
    
    public void setEdge(String edge) {
        this.edge = edge;
    }
}
